############
#Script for creating synthetic LS data from the 2011 SAR

library(plyr)
library(reshape2)
library(ggplot2)

getwd()

SAR2011<-read.csv("../SAR Data/rft-teaching-file/2011 Census Microdata Teaching File.csv")
SAR2011$counter<-1
SingleYearAge<-read.csv("../SAR Data/AgeByRegion/SingleYearAge.csv")

write.csv(SAR2011,"SAR2011.csv")

####
#Age groups in SAR2011 are:
#1 = 0-15
#2 = 16-24
#3 = 25-34
#4 = 35-44
#5 = 45-54
#6 = 55-64
#7 = 65-74
#8 = 75 and over


###########################################

#subset the SAR2011 file by region
#get a vector of the unique regions in the SAR2011 file

#first create a region list from the unique values in SAR2011
regionlist<-as.vector(unique(SAR2011$Region))
regionlist

#length(regionlist)
#regionlist[3]

#this loop then subsets the data frame by regions and stores each subset
#in a new data frame and stores each data frame in a list
i=1
dflist<-list()
for (i in 1:length(regionlist)){
  region<-regionlist[i]
  print(region)
  #assign each dataframe to a list
  dflist[[i]]<-assign(paste0("SARsub_",region),subset(SAR2011,SAR2011$Region==region))
}

###################################################

# ###now calculate the proportions of the total population in each
# ###age group - couple of ways of doing this
# ##first drop the row ids
# SingYrAge<-SingleYearAge[,-1]
# #this way creates array using the transpose function
# SingleYearProps<-t(t(SingYrAge)/colSums(SingYrAge))
# #this way creates a data frame with the sweep function
# SingleYearProps<-sweep(SingYrAge,2,colSums(SingYrAge),'/')
# #check that everything adds up to one
# colSums(SingleYearProps)
# #add the row names back in
# SingleYearProps$row.names<-SingleYearAge$X
# #and re-order
# SingleYearProps<-SingleYearProps[c(12,1,2,3,4,5,6,7,8,9,10,11)]
# 
# SingleYearAge$AgeGrp=0
AgeGrpVector<-c(16,25,35,45,55,65,75,101)

###This is a little loop to the add age groups found in the 
###SAR2011 to single year of age file
i=0
j=1
for (i in 0:nrow(SingleYearAge)){
  print(paste0("i=",i))
  if (i<=AgeGrpVector[j]){
    print(paste0("j=",j))    
    SingleYearAge[i,"AgeGrp"]=j
    i=i+1
  } else {
    j=j+1
    next
  }  
}

#now aggregate single years into age groups
AgeGroups<-ddply(SingleYearAge,.(AgeGrp),summarise,E12000001=sum(E12000001),E12000002=sum(E12000002),E12000003=sum(E12000003),E12000004=sum(E12000004),E12000005=sum(E12000005),E12000006=sum(E12000006),E12000007=sum(E12000007),E12000008=sum(E12000008),E12000009=sum(E12000009),W92000004=sum(W92000004))

write.csv(AgeGroups,"AgeGroups.csv")
write.csv(SingleYearProps2, "SingleYearProps2.csv")
write.csv(SingleYearEmpty, "SingleYearEmpty.csv")

#now calculate the proportion each single year of age comprises of 
#each age group
SingleYearProps2<-SingleYearAge[c(12,2,3,4,5,6,7,8,9,10,11)]
SingleYearEmpty<-SingleYearAge[c(12,2,3,4,5,6,7,8,9,10,11)]

#this loop calculates age group proportions for each age group
i=1
j=1
counter=1
for (i in 1:nrow(SingleYearProps2)){
  if (SingleYearProps2[i,1]==AgeGroups[j,1]){
    SingleYearEmpty[i,2:11]<-SingleYearProps2[i,2:11]/AgeGroups[j,2:11]
    print(paste0("i=",i))
  }else{
    SingleYearEmpty[i,2:11]<-SingleYearProps2[i,2:11]/AgeGroups[j+1,2:11]
    j=j+1
    print(paste0("j=",j))
    #i=i-1
    #print(paste0("i=",i))
    next
  }
}

SingleYearEmpty$row.names<-SingleYearAge$X
SingleYearProps3<-SingleYearEmpty[c(12,1,2,3,4,5,6,7,8,9,10,11)]

sum(SingleYearProps3[,3])

#################################################
############

#create some dummy data frames to store data in 
NumbersInAge<-SingleYearProps3[,-1]
NumbersInAgeRound<-SingleYearProps3[,-1]
Temp<-NumbersInAge
Temp<-NumbersInAgeRound

# #now fill this data frame with estimated numbers of people in each age group
#rounded
i=1
j=1
counter=1
for (i in 1:nrow(Temp)){
  if(Temp[i,1]==SARAgeTotalsDF[j,1]){
    NumbersInAgeRound[i,2:11]<-round(Temp[i,2:11]*SARAgeTotalsDF[j,2:11],0)
    print(paste0("i=",i))
  }else{
    NumbersInAgeRound[i,2:11]<-round(Temp[i,2:11]*SARAgeTotalsDF[j+1,2:11],0)
    j=j+1
    print(paste0("j=",j))
    next
  }
}

# # #now fill this data frame will estimated numbers of people in each age group
# #rounded
# i=1
# j=1
# counter=1
# for (i in 1:nrow(Temp)){
#   if(Temp[i,1]==SARAgeTotalsDF[j,1]){
#     NumbersInAgeRound[i,2:11]<-floor(Temp[i,2:11]*SARAgeTotalsDF[j,2:11])
#     print(paste0("i=",i))
#   }else{
#     NumbersInAgeRound[i,2:11]<-floor(Temp[i,2:11]*SARAgeTotalsDF[j+1,2:11])
#     j=j+1
#     print(paste0("j=",j))
#     next
#   }
# }

#now fill this data frame will estimated numbers of people in each age group
#not rounded
i=1
j=1
counter=1
for (i in 1:nrow(Temp)){
  if(Temp[i,1]==SARAgeTotalsDF[j,1]){
    NumbersInAge[i,2:11]<-Temp[i,2:11]*SARAgeTotalsDF[j,2:11]
    print(paste0("i=",i))
  }else{
    NumbersInAge[i,2:11]<-Temp[i,2:11]*SARAgeTotalsDF[j+1,2:11]
    j=j+1
    print(paste0("j=",j))
    next
  }
}

sum(NumbersInAge[2:11])
sum(SARAgeTotalsDF[2:11])
sum(SAR2011$counter)


###add ID column

NumbersInAgeRound$ID<-seq(0,nrow(NumbersInAgeRound)-1)
NumbersInAge$ID<-seq(0,nrow(NumbersInAge)-1)

write.csv(NumbersInAge,"NumbersInAge.csv")
write.csv(NumbersInAgeRound,"NumbersInAgeRound.csv")


###now check that everything adds up
NumbAgeAgg<-ddply(NumbersInAge,.(AgeGrp),summarise,E12000001=sum(E12000001),E12000002=sum(E12000002),E12000003=sum(E12000003),E12000004=sum(E12000004),E12000005=sum(E12000005),E12000006=sum(E12000006),E12000007=sum(E12000007),E12000008=sum(E12000008),E12000009=sum(E12000009),W92000004=sum(W92000004))
NumbAgeRoundAgg<-ddply(NumbersInAgeRound,.(AgeGrp),summarise,E12000001=sum(E12000001),E12000002=sum(E12000002),E12000003=sum(E12000003),E12000004=sum(E12000004),E12000005=sum(E12000005),E12000006=sum(E12000006),E12000007=sum(E12000007),E12000008=sum(E12000008),E12000009=sum(E12000009),W92000004=sum(W92000004))
#it own't add up due to rounding errors, so a generate rounding error matrix
ErrorMatrix<-NumbAgeAgg-NumbAgeRoundAgg
#use the rounding error matrix to add on or knock off values from each age group
#it's not perfect, but we'll knock off or add on values to the last single age 
#in the age group using the AgeGrpVector from before - crude, but it at least gets us
#to kind of where we want to be...
AgeGrpVector<-c(16,25,35,45,55,65,75,101)

i=1
j=1
k=1
for (i in 1:length(AgeGrpVector)){
  k=AgeGrpVector[i]
  NumbersInAgeRound[k,2:11]<-NumbersInAgeRound[k,2:11]+ErrorMatrix[j,2:11]
  j=j+1
}

AgeGrpVector[1]

########################################################
##Create a data frame of data frames
#cycle through the dataframes in list

summary(dflist)

dataframe=1
for (dataframe in 1:length(dflist)){
  dfname<-regionlist[dataframe]
  df<-as.data.frame(dflist[dataframe])  
  i=1
  j=1
  counter=1 
  for (i in 1:nrow(df)){
    if (counter<=NumbersInAgeRound[j,dfname]){
      df[i,"AgeSingle"]<-NumbersInAgeRound[j,"ID"]
      counter=counter+1
    }else{
      df[i,"AgeSingle"]<-NumbersInAgeRound[j+1,"ID"]
      counter=2
      j=j+1
    }  
  }
  print(paste0("Finished estimation for region ",dfname))
  assign(paste0("SARsub",dfname),df)
}

rm(SARsubE12000001,SARsubE12000002,SARsubE12000003,SARsubE12000004,SARsubE12000005,SARsubE12000006,SARsubE12000007,SARsubE12000008,SARsubE12000009,SARsubW92000004)

###calculate the numbers of people in each age group in the subsets
#SARAgeTotals<-ddply(SAR2011, .(Region, Age), summarise, SumAge=sum(Age))
EstimateCheck<-ddply(SARsubE12000001, .(AgeSingle), nrow)
NumbersInAgeRound[,2]-EstimateCheck[,2]
sum(SARAgeTotals$V1)

#SARsubE12000001$counter<-NULL

#recombine everything back into the original data frame
SAR2011<-rbind(SARsubE12000001,SARsubE12000002,SARsubE12000003,SARsubE12000004,SARsubE12000005,SARsubE12000006,SARsubE12000007,SARsubE12000008,SARsubE12000009,SARsubW92000004)

##now add a new ten year age group variable - use this function to do it:
newvar<-0 
recode_agegroups<-function(original_age_variable){ 
  newvar[original_age_variable >=100]<-10
  newvar[original_age_variable >=90 & original_age_variable <=99]<-9 
  newvar[original_age_variable >=80 & original_age_variable <=89]<-8
  newvar[original_age_variable >=70 & original_age_variable <=79]<-7 
  newvar[original_age_variable >=60 & original_age_variable <=69]<-6 
  newvar[original_age_variable >=50 & original_age_variable <=59]<-5  
  newvar[original_age_variable >=40 & original_age_variable <=49]<-4 
  newvar[original_age_variable >=30 & original_age_variable <=39]<-3 
  newvar[original_age_variable >=20 & original_age_variable <=29]<-2
  newvar[original_age_variable >=10 & original_age_variable <=19]<-1 
  newvar[original_age_variable >=0 & original_age_variable <=9]<-0  
  return(newvar) 
}

SAR2011$Age10YrGrp<-recode_agegroups(SAR2011$AgeSingle)

SAR2011[97500,]


####################################################################
####################################################################
###Variable Estimation

#before going any further bung in a pseuso ID column for reordering later on...
SAR2011$Pseudo_ID<-seq(1,nrow(SAR2011))

####################################################################
####################################################################
###Health

sub_health<-health_trans[,1:3]
health_matrix<-melt(health_trans, id=c("Health_2011","Health_2001"))
health_matrix<-dcast(sub_health, Health_2011~Health_2001)
health_matrix<-health_matrix[1:4,]

transitionPlot(health_matrix)

#read in the health transitions
health_trans<-read.csv("Health.csv")

health_trans_totals<-health_trans
health_trans_totals_round<-health_trans

#First we need to calculate numbers of people by health status and age
#in the SAR file:
SARHealthTotals<-ddply(SAR2011, .(Age10YrGrp, Health), nrow)
#re-cast back into a square format:
SARHealthTotalsDF<-dcast(SARHealthTotals,Health~Age10YrGrp)
SARHealthTotalsDF<-SARHealthTotalsDF[2:6,c(1,3:11)]

write.csv(SARHealthTotalsDF,"SARHealthTotalsDF.csv")

###
#calculate the numbers of people undergoing each health transition

i=1
k=1
for(i in 1:nrow(health_trans)){
  k<-health_trans[i,1]
  health_trans_totals[i,3:11]<-health_trans[i,3:11]*SARHealthTotalsDF[k,2:10]
}

i=1
k=1
for(i in 1:nrow(health_trans)){
  k<-health_trans[i,1]
  health_trans_totals_round[i,3:11]<-round(health_trans[i,3:11]*SARHealthTotalsDF[k,2:10],0)
}

#aggregate back into a matrix the same shape as SARHealthTotals
health_trans_agg<-ddply(health_trans_totals_round,.(Health_2011),summarise,X1=sum(X1),X2=sum(X2),X3=sum(X3),X4=sum(X4),X5=sum(X5),X6=sum(X6),X7=sum(X7),X8=sum(X8),X9=sum(X9))
  

######now generate an error matrix
ErrorMatrix<-SARHealthTotalsDF-health_trans_agg

#
HealthGrpVector<-c(1,4,7,10,13)

#now use the error matrix to update health_trans_totals_round
i=1
j=1
k=1
for (i in 1:length(HealthGrpVector)){
  k=HealthGrpVector[i]
  health_trans_totals_round[k,3:11]<-health_trans_totals_round[k,3:11]+ErrorMatrix[j,2:10]
  j=j+1
}

#check that you've updated correctly - error matrix should be full of zeros
health_trans_agg1<-ddply(health_trans_totals_round,.(Health_2011),summarise,X1=sum(X1),X2=sum(X2),X3=sum(X3),X4=sum(X4),X5=sum(X5),X6=sum(X6),X7=sum(X7),X8=sum(X8),X9=sum(X9))
ErrorMatrix<-SARHealthTotalsDF-health_trans_agg1

write.csv(health_trans_totals_round, "health_count.csv")

#rename columns
rename(health_trans_totals_round,c("X1"="1","X2"="2","X3"="3","X4"="4","X5"="5","X6"="6","X7"="7","X8"="8","X9"="9"))

#chop up the SAR file into chunks by health and age group
#first create an age group list from the unique values in SAR2011
AgeGrpList<-as.vector(unique(SAR2011$Age10YrGrp))
#AgeGrpList<-c("X1","X2","X3","X4","X5","X6","X7","X8","X9","10")

#this loop then subsets the data frame by age groups and stores each 
#subset in a new data frame and stores each data frame in a list
i=1
AgeGrpDFList<-list()
for (i in 1:length(AgeGrpList)){
  AgeGrp<-AgeGrpList[i]
  print(AgeGrp)
  #create the age group subset
  temp_df<-subset(SAR2011,SAR2011$Age10YrGrp==AgeGrp)
  #ramdomise the order of the subset
  temp_df<-temp_df[sample(1:nrow(temp_df)),]
  #now put the missing data (-9) values at the top of the file
  temp_df<-arrange(temp_df,Health)
  AgeGrpDFList[[i]]<-assign(paste0("Age10YrGrp_",AgeGrp),temp_df)
}

#we're going to need a method to get rid of the -9 data!
#Age10YrGrp_1<-Age10YrGrp_1[sample(1:nrow(Age10YrGrp_1)),]
#Age10YrGrp_1<-arrange(Age10YrGrp_1,Health)

#Age10YrGrp_1a<-subset(Age10YrGrp_1,Age10YrGrp_1$Health==-9)
#Age10YrGrp_1b<-subset(Age10YrGrp_1,Age10YrGrp_1$Health!=-9)

##############
#Estimation loop

dataframe=1
for (dataframe in 3:length(AgeGrpDFList)-1){
  dfname<-AgeGrpList[dataframe]
  df<-as.data.frame(AgeGrpDFList[dataframe])  
  i=1
  j=1
  counter=1 
  for (i in 1:nrow(df)){
    if (df[i,"Health"]==-9){
      df[i,"Health2001"]<--9
      print(i)
      next
    } else {
      print(paste0("starting next loop - counter is: ",counter))      
      if (counter<=health_trans_totals_round[j,dfname+2]){
        df[i,"Health2001"]<-health_trans_totals_round[j,"Health_2001"]
        counter=counter+1
      }else{
        df[i,"Health2001"]<-health_trans_totals_round[j+1,"Health_2001"]
        counter=2
        j=j+1
      }  
    }    
  }
  print(paste0("Finished estimation for age group ",dfname))
  assign(paste0("AgeHealthSub",dfname),df)
}

##add a new Health2001 column onto the unused dataframes

Age10YrGrp_0$Health2001<--7
Age10YrGrp_10$Health2001<--7

#recombine everything back into a single dataframe
SAR2011<-rbind(Age10YrGrp_0,AgeHealthSub1,AgeHealthSub2,AgeHealthSub3,AgeHealthSub4,AgeHealthSub5,AgeHealthSub6,AgeHealthSub7,AgeHealthSub8,AgeHealthSub9,Age10YrGrp_10)

#reorder by pesudo_id
SAR2011<-arrange(SAR2011,Pseudo_ID)

write.csv(SAR2011,"SAR2011.csv")
write.csv(df,"df.csv")

SAR2011$Health2001<-replace(SAR2011$Health2001,"NA",-9)

#########################################################################
########################################################################
#Marital Status

marital_trans<-read.csv("Marital.csv", header=TRUE)
marital_trans_totals <- marital_trans
marital_trans_totals_round <- marital_trans

##calculate the marital status totals
SARMaritalTotals<-ddply(SAR2011, .(Age10YrGrp, Marital.Status), nrow)

##cast back into a square matrix
SARMaritalTotalsDF<-dcast(SARMaritalTotals,Marital.Status~Age10YrGrp)
SARMaritalTotalsDF<-SARMaritalTotalsDF[,c(1,3:11)]
####################
###
#calculate the numbers of people undergoing each marital transition

i=1
k=1
for(i in 1:nrow(marital_trans)){
  k<-marital_trans[i,1]
  marital_trans_totals[i,3:11]<-marital_trans[i,3:11]*SARMaritalTotalsDF[k,2:10]
}

i=1
k=1
for(i in 1:nrow(marital_trans)){
  k<-marital_trans[i,1]
  marital_trans_totals_round[i,3:11]<-round(marital_trans[i,3:11]*SARMaritalTotalsDF[k,2:10],0)
}

#aggregate back into a matrix the same shape as SARMaritalTotals
marital_trans_agg<-ddply(marital_trans_totals_round,.(Marital_2011),summarise,X1=sum(X1),X2=sum(X2),X3=sum(X3),X4=sum(X4),X5=sum(X5),X6=sum(X6),X7=sum(X7),X8=sum(X8),X9=sum(X9))


######now generate an error matrix
ErrorMatrix<-SARMaritalTotalsDF-marital_trans_agg

###OK a few big-ish errors here. Can't remember why bit it might be due
#to some small numbers in the LS and us adjusting them to 0? Well, let's
#knock 'em off anyway...

MaritalGrpVector<-c(1,6,11,16,21)

#now use the error matrix to update marital_trans_totals_round
i=1
j=1
k=1
for (i in 1:length(MaritalGrpVector)){
  k=MaritalGrpVector[i]
  marital_trans_totals_round[k,3:11]<-marital_trans_totals_round[k,3:11]+ErrorMatrix[j,2:10]
  j=j+1
}

#check that you've updated correctly - error matrix should be full of zeros
marital_trans_agg1<-ddply(marital_trans_totals_round,.(Marital_2011),summarise,X1=sum(X1),X2=sum(X2),X3=sum(X3),X4=sum(X4),X5=sum(X5),X6=sum(X6),X7=sum(X7),X8=sum(X8),X9=sum(X9))
ErrorMatrix<-SARMaritalTotalsDF-marital_trans_agg1

#there is a little error in there - edit manually. 
marital_trans_totals_round<-edit(marital_trans_totals_round)

#chop SAR2011 up into age groups, randomise and put -9s at the top 
i=1
AgeGrpDFList<-list()
for (i in 1:length(AgeGrpList)){
  AgeGrp<-AgeGrpList[i]
  print(AgeGrp)
  #create the age group subset
  temp_df<-subset(SAR2011,SAR2011$Age10YrGrp==AgeGrp)
  #ramdomise the order of the subset
  temp_df<-temp_df[sample(1:nrow(temp_df)),]
  #now put the missing data (-9) values at the top of the file
  temp_df<-arrange(temp_df,Marital.Status)
  AgeGrpDFList[[i]]<-assign(paste0("Age10YrGrp_",AgeGrp),temp_df)
}


#################################################################
#Estimation loop

############
################################################################
#create function to create a vector for the rows of the matrix 
#with data in

indexVectorCreator<-function(transitionMatrix,column){
  index<-vector()
  i=1  
  for (i in 1:nrow(transitionMatrix)){
    if (transitionMatrix[i,column]!=0){
      index[[i]]<-i
    } else {
      next
    }
  }
  index<-index[!is.na(index)]
  return(index)
}

index<-indexVectorCreator(marital_trans_totals_round,5)
#################################################################
################

dataframe=1
for (dataframe in 3:length(AgeGrpDFList)-1){
  #create a little index vector so that the programme knows to
  #ignore 0s in the loop  
  dfname<-AgeGrpList[dataframe]
  df<-as.data.frame(AgeGrpDFList[dataframe])  
  i=1
  j=1
  counter=1  
  for (i in 1:nrow(df)){    
    if (df[i,"Marital.Status"]==-9){
      df[i,"Marital2001"]<--9
      print(i)
      next
    } else {
      print(paste0("Sub-dataframe ",j," group number ",indexcounter," row ",counter))
      index<-indexVectorCreator(marital_trans_totals_round,dfname+2)
      indexcounter<-index[[j]]
      if (counter<=marital_trans_totals_round[indexcounter,dfname+2]){
        df[i,"Marital2001"]<-marital_trans_totals_round[indexcounter,"Marital_2001"]
        counter=counter+1
      } else {
        indexcounter<-index[[j+1]]
        df[i,"Marital2001"]<-marital_trans_totals_round[indexcounter,"Marital_2001"]
        counter=2
        j=j+1
      }
    }    
  }
  print(paste0("Finished estimation for marital group ",dfname))
  assign(paste0("AgeMaritalSub",dfname),df)
}

Age10YrGrp_0$Marital2001<--7
Age10YrGrp_10$Marital2001<--7

#recombine everything back into a single dataframe
SAR2011<-rbind(Age10YrGrp_0,AgeMaritalSub1,AgeMaritalSub2,AgeMaritalSub3,AgeMaritalSub4,AgeMaritalSub5,AgeMaritalSub6,AgeMaritalSub7,AgeMaritalSub8,AgeMaritalSub9,Age10YrGrp_10)

#reorder by pesudo_id
SAR2011<-arrange(SAR2011,Pseudo_ID)

write.csv(SAR2011,"SAR2011.csv")

#####now check everything adds up...
##calculate the marital status totals
SARMarital2001Totals<-ddply(SAR2011, .(Age10YrGrp, Marital2001), nrow)

##cast back into a square matrix
SARMarital2001TotalsDF<-dcast(SARMarital2001Totals,Marital2001~Age10YrGrp)
SARMarital2001TotalsDF<-SARMarital2001TotalsDF[,c(1,3:11)]

write.csv(marital_trans_totals_round,"marital_trans_totals_round.csv")
write.csv(SARMarital2001TotalsDF,"SARMarital2001TotalsDF.csv")

#########################################################################
########################################################################
#Religion

#little function to recode religion into missing, no religion and religion

newvar<-0 
recode_religion<-function(original_religion_variable){ 
  newvar[original_religion_variable ==-9]<--9
  newvar[original_religion_variable ==1]<-1 
  newvar[original_religion_variable ==2]<-2
  newvar[original_religion_variable ==3]<-2 
  newvar[original_religion_variable ==4]<-2 
  newvar[original_religion_variable ==5]<-2  
  newvar[original_religion_variable ==6]<-2 
  newvar[original_religion_variable ==7]<-2 
  newvar[original_religion_variable ==8]<-2
  newvar[original_religion_variable ==9]<-2 
  return(newvar) 
}

#now create a new variable in the SAR2011 dataset which is the re-coded
#religion data
SAR2011$ReligionBinary<-recode_religion(SAR2011a$Religion)

##calculate the binary religion totals
SARReligionTotals<-ddply(SAR2011a, .(Age10YrGrp, ReligionBinary), nrow)

##cast back into a square matrix
SARReligionTotalsDF<-dcast(SARReligionTotals,ReligionBinary~Age10YrGrp)
SARReligionTotalsDF<-SARReligionTotalsDF[2:6,c(1,3:11)]

###
#OK, the next task is to estimate the number of religion changers 
#from those who've got a religion in 2011

#chop SAR2011 up into age groups, randomise and put -9s at the top 
i=1
AgeGrpDFList<-list()
for (i in 1:length(AgeGrpList)){
  AgeGrp<-AgeGrpList[i]
  print(AgeGrp)
  #create the age group subset
  temp_df<-subset(SAR2011,SAR2011$Age10YrGrp==AgeGrp)
  #ramdomise the order of the subset
  temp_df<-temp_df[sample(1:nrow(temp_df)),]
  #now put the missing data (-9) values at the top of the file
  temp_df<-arrange(temp_df,ReligionBinary)
  AgeGrpDFList[[i]]<-assign(paste0("Age10YrGrp_",AgeGrp),temp_df)
}

##Step 1

#we are now going to need to recode our recoded religion variable so that
#some of the people coded as 2 - religion, are recoded into 3 - different
#religion. We know how many this should be from the LS transition data.
#I calculated these numbers externally, so now read in the matrix...

relig_recode<-read.csv("ReligionRecode.csv")

dataframe=1
for (dataframe in 3:length(AgeGrpDFList)-1){
  dfname<-AgeGrpList[dataframe]
  df<-as.data.frame(AgeGrpDFList[dataframe])  
  i=1
  j=3
  counter=1 
  for (i in 1:nrow(df)){
    if (df[i,"ReligionBinary"]==-9){
      df[i,"ReligionTernary"]<--9
      #print(i)
      next
    } else if (df[i,"ReligionBinary"]==1){
      df[i,"ReligionTernary"]<-1
      #print(i)
      next
    } else {
      print(paste0("starting next loop - counter is: ",counter))      
      if (counter<=relig_recode[j,dfname+2]){
        df[i,"ReligionTernary"]<-relig_recode[j,"Religion_2011"]
        counter=counter+1
      } else {
        df[i,"ReligionTernary"]<-relig_recode[j+1,"Religion_2011"]
        counter=2
        j=j+1
      }  
    }    
  }
  print(paste0("Finished estimation for ReligionTernary ",dfname))
  assign(paste0("ReligSub",dfname),df)
}

#check it's done things as expected...
write.csv(ReligSub2, "religsub1.csv")

#Step 2 - Now for the backward estimation of religion from the 2011
#recoded stuff

#first re-combine all of the new estimates back into single dataframe,
#split up and fill the dataframe of dataframes again...

Age10YrGrp_0$ReligionTernary<--7
Age10YrGrp_10$ReligionTernary<--7

#recombine everything back into a single dataframe
SAR2011<-rbind(Age10YrGrp_0,ReligSub1,ReligSub2,ReligSub3,ReligSub4,ReligSub5,ReligSub6,ReligSub7,ReligSub8,ReligSub9,Age10YrGrp_10)

#reorder by pesudo_id
SAR2011<-arrange(SAR2011,Pseudo_ID)

##calculate the ternary religion totals
SARReligionTernTotals<-ddply(SAR2011, .(Age10YrGrp, ReligionTernary), nrow)

##cast back into a square matrix
SARReligionTernTotalsDF<-dcast(SARReligionTernTotals,ReligionTernary~Age10YrGrp)
SARReligionTernTotalsDF<-SARReligionTernTotalsDF[3:5,c(1,3:11)]

######
#read in the religion transition probabilities

religion_trans<-read.csv("Religion.csv")

religion_trans_totals<-religion_trans
religion_trans_totals_round<-religion_trans

#
write.csv(SARReligionTernTotalsDF,"SARReligionTernTotalsDF.csv")

###
#calculate the numbers of people undergoing each religious transition

i=1
k=1
for(i in 1:nrow(religion_trans)){
  k<-religion_trans[i,1]
  religion_trans_totals[i,3:11]<-religion_trans[i,3:11]*SARReligionTernTotalsDF[k,2:10]
}

i=1
k=1
for(i in 1:nrow(religion_trans)){
  k<-religion_trans[i,1]
  religion_trans_totals_round[i,3:11]<-round(religion_trans[i,3:11]*SARReligionTernTotalsDF[k,2:10],0)
}

#aggregate back into a matrix the same shape as SARHealthTotals
religion_trans_agg<-ddply(religion_trans_totals_round,.(Religion_2011),summarise,X1=sum(X1),X2=sum(X2),X3=sum(X3),X4=sum(X4),X5=sum(X5),X6=sum(X6),X7=sum(X7),X8=sum(X8),X9=sum(X9))


######now generate an error matrix
ErrorMatrix<-SARReligionTernTotalsDF-religion_trans_agg
###ooh, looks like no errors, so we can move on to the estimation step!

#chop SAR2011 up into age groups, randomise and put -9s at the top 
i=1
AgeGrpDFList<-list()
for (i in 1:length(AgeGrpList)){
  AgeGrp<-AgeGrpList[i]
  print(AgeGrp)
  #create the age group subset
  temp_df<-subset(SAR2011,SAR2011$Age10YrGrp==AgeGrp)
  #ramdomise the order of the subset
  temp_df<-temp_df[sample(1:nrow(temp_df)),]
  #now put the missing data (-9) values at the top of the file
  temp_df<-arrange(temp_df,ReligionTernary)
  AgeGrpDFList[[i]]<-assign(paste0("Age10YrGrp_",AgeGrp),temp_df)
}

#################################################################
#Estimation loop

dataframe=1
for (dataframe in 3:length(AgeGrpDFList)-1){
  dfname<-AgeGrpList[dataframe]
  df<-as.data.frame(AgeGrpDFList[dataframe])  
  i=1
  j=1
  counter=1 
  for (i in 1:nrow(df)){
    if (df[i,"ReligionTernary"]==-9){
      df[i,"Religion2001"]<--9
      print(i)
      next
    } else {
      print(paste0("starting next loop - counter is: ",counter))      
      if (counter<=religion_trans_totals_round[j,dfname+2]){
        df[i,"Religion2001"]<-religion_trans_totals_round[j,"Religion_2001"]
        counter=counter+1
      }else{
        df[i,"Religion2001"]<-religion_trans_totals_round[j+1,"Religion_2001"]
        counter=2
        j=j+1
      }  
    }    
  }
  print(paste0("Finished estimation for religion group ",dfname))
  assign(paste0("AgeReligionSub",dfname),df)
}

Age10YrGrp_0$Religion2001<--9
Age10YrGrp_10$Religion2001<--9

#recombine everything back into a single dataframe
SAR2011<-rbind(Age10YrGrp_0,AgeReligionSub1,AgeReligionSub2,AgeReligionSub3,AgeReligionSub4,AgeReligionSub5,AgeReligionSub6,AgeReligionSub7,AgeReligionSub8,AgeReligionSub9,Age10YrGrp_10)

#reorder by pesudo_id
SAR2011<-arrange(SAR2011,Pseudo_ID)

write.csv(SAR2011,"SAR2011.csv")

##calculate the 2001 religion totals
SARReligion2001Totals<-ddply(SAR2011, .(Age10YrGrp, Religion2001), nrow)

##cast back into a square matrix
SARReligion2001TotalsDF<-dcast(SARReligion2001Totals,Religion2001~Age10YrGrp)
SARReligion2001TotalsDF<-SARReligion2001TotalsDF[3:5,c(1,3:11)]




#########################################################################
########################################################################
#Social Grade

##*notes
#For social grade we have some dodgy transitions in the 10-19 and 
#90-99 age groups - therefore I've borrowed the transitional probs
#from the neighbouring age groups - 10-19 is the same as 20-29 and 
#90-99 is the same as 80-89...

social_trans<-read.csv("Social.csv", header=TRUE)
social_trans_totals <- social_trans
social_trans_totals_round <- social_trans

##calculate the social status totals
SARSocialTotals<-ddply(SAR2011, .(Age10YrGrp, Approximated.Social.Grade), nrow)

##cast back into a square matrix
SARSocialTotalsDF<-dcast(SARSocialTotals,Approximated.Social.Grade~Age10YrGrp)
SARSocialTotalsDF<-SARSocialTotalsDF[2:5,c(1,3:11)]

write.csv(SARSocialTotalsDF, "SARSocialTotalsDF.csv")

####################
###
#calculate the numbers of people undergoing each social transition

i=1
k=1
for(i in 1:nrow(social_trans)){
  k<-social_trans[i,1]
  social_trans_totals[i,3:11]<-social_trans[i,3:11]*SARSocialTotalsDF[k,2:10]
}

i=1
k=1
for(i in 1:nrow(social_trans)){
  k<-social_trans[i,1]
  social_trans_totals_round[i,3:11]<-round(social_trans[i,3:11]*SARSocialTotalsDF[k,2:10],0)
}

#aggregate back into a matrix the same shape as SARSocialTotals
social_trans_agg<-ddply(social_trans_totals_round,.(Social_2011),summarise,X1=sum(X1),X2=sum(X2),X3=sum(X3),X4=sum(X4),X5=sum(X5),X6=sum(X6),X7=sum(X7),X8=sum(X8),X9=sum(X9))


######now generate an error matrix
ErrorMatrix<-SARSocialTotalsDF-social_trans_agg

###OK a few big-ish errors here. Can't remember why bit it might be due
#to some small numbers in the LS and us adjusting them to 0? Well, let's
#knock 'em off anyway...

SocialGrpVector<-c(1,5,9,13)

#now use the error matrix to update social_trans_totals_round
i=1
j=1
k=1
for (i in 1:length(SocialGrpVector)){
  k=SocialGrpVector[i]
  social_trans_totals_round[k,3:11]<-social_trans_totals_round[k,3:11]+ErrorMatrix[j,2:10]
  j=j+1
}

#check that you've updated correctly - error matrix should be full of zeros
social_trans_agg1<-ddply(social_trans_totals_round,.(Social_2011),summarise,X1=sum(X1),X2=sum(X2),X3=sum(X3),X4=sum(X4),X5=sum(X5),X6=sum(X6),X7=sum(X7),X8=sum(X8),X9=sum(X9))
ErrorMatrix<-SARSocialTotalsDF-social_trans_agg1

#if there is a little error in there - edit manually. 
social_trans_totals_round<-edit(social_trans_totals_round)

#chop SAR2011 up into age groups, randomise and put -9s at the top 
i=1
AgeGrpDFList<-list()
for (i in 1:length(AgeGrpList)){
  AgeGrp<-AgeGrpList[i]
  print(AgeGrp)
  #create the age group subset
  temp_df<-subset(SAR2011,SAR2011$Age10YrGrp==AgeGrp)
  #ramdomise the order of the subset
  temp_df<-temp_df[sample(1:nrow(temp_df)),]
  #now put the missing data (-9) values at the top of the file
  temp_df<-arrange(temp_df,Approximated.Social.Grade)
  AgeGrpDFList[[i]]<-assign(paste0("Age10YrGrp_",AgeGrp),temp_df)
}


#################################################################
#Estimation loop
############

################################################################
#a function to create a vector idenfifying the rows of the matrix 
#with non-zeros data in

indexVectorCreator<-function(transitionMatrix,column){
  index<-vector()
  i=1  
  for (i in 1:nrow(transitionMatrix)){
    if (transitionMatrix[i,column]!=0){
      index[[i]]<-i
    } else {
      next
    }
  }
  index<-index[!is.na(index)]
  return(index)
}

dataframe=1
for (dataframe in 3:length(AgeGrpDFList)-1){
  #create a little index vector so that the programme knows to
  #ignore 0s in the loop  
  dfname<-AgeGrpList[dataframe]
  df<-as.data.frame(AgeGrpDFList[dataframe])  
  i=1
  j=1
  counter=1  
  for (i in 1:nrow(df)){    
    if (df[i,"Approximated.Social.Grade"]==-9){
      df[i,"Social2001"]<--9
      print(i)
      next
    } else {
      print(paste0("Sub-dataframe ",j," group number ",indexcounter," row ",counter))
      index<-indexVectorCreator(social_trans_totals_round,dfname+2)
      indexcounter<-index[[j]]
      if (counter<=social_trans_totals_round[indexcounter,dfname+2]){
        df[i,"Social2001"]<-social_trans_totals_round[indexcounter,"Social_2001"]
        counter=counter+1
      } else {
        indexcounter<-index[[j+1]]
        df[i,"Social2001"]<-social_trans_totals_round[indexcounter,"Social_2001"]
        counter=2
        j=j+1
      }
    }    
  }
  print(paste0("Finished estimation for social group ",dfname))
  assign(paste0("AgeSocialSub",dfname),df)
}

Age10YrGrp_0$Social2001<--7
Age10YrGrp_10$Social2001<--7

#recombine everything back into a single dataframe
SAR2011<-rbind(Age10YrGrp_0,AgeSocialSub1,AgeSocialSub2,AgeSocialSub3,AgeSocialSub4,AgeSocialSub5,AgeSocialSub6,AgeSocialSub7,AgeSocialSub8,AgeSocialSub9,Age10YrGrp_10)

#reorder by pesudo_id
SAR2011<-arrange(SAR2011,Pseudo_ID)

write.csv(SAR2011,"SAR2011.csv")

#####now check everything adds up...
##calculate the social status totals
SARSocial2001Totals<-ddply(SAR2011, .(Age10YrGrp, Social2001), nrow)

##cast back into a square matrix
SARSocial2001TotalsDF<-dcast(SARSocial2001Totals,Social2001~Age10YrGrp)
SARSocial2001TotalsDF<-SARSocial2001TotalsDF[,c(1,3:11)]

write.csv(social_trans_totals_round,"social_trans_totals_round.csv")
write.csv(SARSocial2001TotalsDF,"SARSocial2001TotalsDF.csv")


#########################################################################
########################################################################
#Births

births<-read.csv("Births.csv")

unique(SAR2011$Sex)
SARMaleSub<-subset(SAR2011,SAR2011$Sex==1)
SARFemaleSub<-subset(SAR2011,SAR2011$Sex==2)

#chop SARFemaleSub up into age groups, randomise and put -9s at the top 
i=1
AgeGrpDFList<-list()
for (i in 1:length(AgeGrpList)){
  AgeGrp<-AgeGrpList[i]
  print(AgeGrp)
  #create the age group subset
  temp_df<-subset(SARFemaleSub,SARFemaleSub$Age10YrGrp==AgeGrp)
  #ramdomise the order of the subset
  temp_df<-temp_df[sample(1:nrow(temp_df)),]
  #now put the missing data (-9) values at the top of the file
  temp_df<-arrange(temp_df,Sex)
  AgeGrpDFList[[i]]<-assign(paste0("Fem10YrGrp_",AgeGrp),temp_df)
}

##calculate the social status totals
SARFemaleAgeTotals<-ddply(SARFemaleSub, .(Age10YrGrp, Sex), nrow)

SARFemaleTotalsDF<-dcast(SARFemaleAgeTotals, Sex~Age10YrGrp, sum)
SARFemaleTotalsDF<-SARFemaleTotalsDF[,c(1,3:7)]

check<-SARFemaleTotalsDF
check[1,2:6]<-0

births_totals<-births
births_totals_round<-births

#calculate the total numbers of births expected to women in each age group
i=1
k=1
for(i in 1:nrow(births)){
  k<-births[i,1]+1
  births_totals[i,2:6]<-births[i,2:6]*SARFemaleTotalsDF[1,2:6]
}


i=1
k=1
for(i in 1:nrow(births)){
  k<-births[i,1]+1
  births_totals_round[i,2:6]<-round(births[i,2:6]*SARFemaleTotalsDF[1,2:6],0)
}

###############################################

row=1
for (row in 1:nrow(births_totals_round)){
  check[1,2:6]<-check[1,2:6]+births_totals_round[row,2:6]
}


######now generate an error matrix
ErrorMatrix<-SARFemaleTotalsDF-check

#manual edit time:
births_totals_round<-edit(births_totals_round)


#Right, now we're ready to go!!

dataframe=2
for (dataframe in 2:6){
  #create a little index vector so that the programme knows to
  #ignore 0s in the loop  
  dfname<-AgeGrpList[dataframe]
  df<-as.data.frame(AgeGrpDFList[dataframe])  
  i=1
  j=1
  counter=1  
  for (i in 1:nrow(df)){    
    if (df[i,"Sex"]==-9){
      df[i,"Births2001_to_2011"]<--9
      print(i)
      next
    } else {
      print(paste0("Sub-dataframe ",j," group number ",indexcounter," row ",counter))
      index<-indexVectorCreator(births_totals_round,dfname+1)
      indexcounter<-index[[j]]
      if (counter<=births_totals_round[indexcounter,dfname+1]){
        df[i,"Births2001_to_2011"]<-births_totals_round[indexcounter,"Births_2001_to_2011"]
        counter=counter+1
      } else {
        indexcounter<-index[[j+1]]
        df[i,"Births2001_to_2011"]<-births_totals_round[indexcounter,"Births_2001_to_2011"]
        counter=2
        j=j+1
      }
    }    
  }
  print(paste0("Finished estimation for births ",dfname))
  assign(paste0("AgeBirthsSub",dfname),df)
}

Fem10YrGrp_0$Births2001_to_2011<-0
Fem10YrGrp_6$Births2001_to_2011<-0
Fem10YrGrp_7$Births2001_to_2011<-0
Fem10YrGrp_8$Births2001_to_2011<-0
Fem10YrGrp_9$Births2001_to_2011<-0
Fem10YrGrp_10$Births2001_to_2011<-0
SARMaleSub$Births2001_to_2011<-0

#recombine everything back into a single dataframe
temp_df<-rbind(Fem10YrGrp_0,AgeBirthsSub1,AgeBirthsSub2,AgeBirthsSub3,AgeBirthsSub4,AgeBirthsSub5,Fem10YrGrp_6,Fem10YrGrp_7,Fem10YrGrp_8,Fem10YrGrp_9,Fem10YrGrp_10,SARMaleSub)

#reorder by pesudo_id
temp_df<-arrange(temp_df,Pseudo_ID)
SAR2011<-temp_df

write.csv(SAR2011,"SAR2011.csv")

#########################################################################
########################################################################
#Deaths

#Estimate deaths by age group by calculating number of deaths by 2001 health status

death_trans<-read.csv("Deaths.csv", header=TRUE)
death_trans<-death_trans[,1:11]
death_trans_totals <- death_trans
death_trans_totals_round <- death_trans

##calculate the 2001 Health status totals
SAR2001HealthTotals<-ddply(SAR2011, .(Age10YrGrp, Health2001), nrow)

##cast back into a square matrix
SAR2001HealthTotalsDF<-dcast(SAR2001HealthTotals,Health2001~Age10YrGrp)
SAR2001HealthTotalsDF<-SAR2001HealthTotalsDF[3:5,c(1,3:11)]


####################
###
#calculate the numbers of people undergoing each social transition

i=1
k=1
for(i in 1:nrow(death_trans)){
  k<-death_trans[i,2]
  death_trans_totals[i,3:11]<-death_trans[i,3:11]*SAR2001HealthTotalsDF[k,2:10]
}

i=1
k=1
for(i in 1:nrow(death_trans)){
  k<-death_trans[i,2]
  death_trans_totals_round[i,3:11]<-round(death_trans[i,3:11]*SAR2001HealthTotalsDF[k,2:10],0)
}

#aggregate back into a matrix the same shape as SARSocialTotals
death_trans_agg<-ddply(death_trans_totals_round,.(Health_2001),summarise,X1=sum(X1),X2=sum(X2),X3=sum(X3),X4=sum(X4),X5=sum(X5),X6=sum(X6),X7=sum(X7),X8=sum(X8),X9=sum(X9))


######now generate an error matrix
ErrorMatrix<-SAR2001HealthTotalsDF-death_trans_agg

write.csv(death_trans_agg,"death_trans_agg.csv")
write.csv(death_trans_totals_round,"death_trans_totals_round.csv")

##Excellent, no errors!!

#chop SAR2011 up into age groups, randomise and put -9s at the top 
i=1
AgeGrpDFList<-list()
for (i in 1:length(AgeGrpList)){
  AgeGrp<-AgeGrpList[i]
  print(AgeGrp)
  #create the age group subset
  temp_df<-subset(SAR2011,SAR2011$Age10YrGrp==AgeGrp)
  #ramdomise the order of the subset
  temp_df<-temp_df[sample(1:nrow(temp_df)),]
  #now put the missing data (-9) values at the top of the file
  temp_df<-arrange(temp_df,Health2001)
  AgeGrpDFList[[i]]<-assign(paste0("Age10YrGrp_",AgeGrp),temp_df)
}


#################################################################
#Estimation loop

dataframe=2
for (dataframe in 3:length(AgeGrpDFList)-1){
  #create a little index vector so that the programme knows to
  #ignore 0s in the loop  
  dfname<-AgeGrpList[dataframe]
  df<-as.data.frame(AgeGrpDFList[dataframe])  
  i=1
  j=1
  counter=1  
  for (i in 1:nrow(df)){    
    if (df[i,"Health2001"]==-9){
      df[i,"AliveOrDead2011"]<--9
      print(i)
      next
    } else {
      print(paste0("Sub-dataframe ",j," group number ",indexcounter," row ",counter))
      index<-indexVectorCreator(death_trans_totals_round,dfname+2)
      indexcounter<-index[[j]]
      if (counter<=death_trans_totals_round[indexcounter,dfname+2]){
        df[i,"AliveOrDead2011"]<-death_trans_totals_round[indexcounter,"AliveOrDead2011"]
        counter=counter+1
      } else {
        indexcounter<-index[[j+1]]
        df[i,"AliveOrDead2011"]<-death_trans_totals_round[indexcounter,"AliveOrDead2011"]
        counter=2
        j=j+1
      }
    }    
  }
  print(paste0("Finished estimation for dead or alive group ",dfname))
  assign(paste0("AgeDeathSub",dfname),df)
}

Age10YrGrp_0$AliveOrDead2011<--7
Age10YrGrp_10$AliveOrDead2011<--7

#recombine everything back into a single dataframe
SAR2011a<-rbind(Age10YrGrp_0,AgeDeathSub1,AgeDeathSub2,AgeDeathSub3,AgeDeathSub4,AgeDeathSub5,AgeDeathSub6,AgeDeathSub7,AgeDeathSub8,AgeDeathSub9,Age10YrGrp_10)

#reorder by pesudo_id
SAR2011a<-arrange(SAR2011a,Pseudo_ID)

write.csv(SAR2011a,"SAR2011a.csv")
SAR2011a<-read.csv("SAR2011a.csv")

#######################################################################
#validation and cross checking...


##calculate the 2001 Health status totals
SAR2001HealthMarriageTotals<-ddply(SAR2011a, .(Health, Marital2001, Marital.Status), nrow)

##cast back into a square matrix
SAR2001HealthTotalsDF<-dcast(SAR2001HealthTotals,Health2001~Age10YrGrp)
SAR2001HealthTotalsDF<-SAR2001HealthTotalsDF[3:5,c(1,3:11)]

